@@ -17,35 +17,24 @@
  *
  * @brief Wallpaper change dialog
  *
- * Program to change wallpapers.
+ * Wallpaper changing appliation.
  *
- * @date November 4, 2019
+ * @date November 7, 2019
  *
- * @version 1.0.1
+ * @version 1.1
  * 
  * @author Michał Bąbik <michalb1981@o2.pl>
  */
 #include <stdio.h> 
-#include <dirent.h> 
 #include <gtk/gtk.h>
 #include "setts.h"
+#include "settstr.h"
 #include "iminfo.h"
 #include "wallset.h"
 #include "imgs.h"
 #include "dlgs.h"
+#include "treev.h"
 /*----------------------------------------------------------------------------*/
-/**
- * @brief  TreeView Columns
- */
-enum {
-    COL_FULL_FILE_NAME,  /**< Full file name with path */
-    COL_FILE_NAME,       /**< Only file name */
-    COL_FILE_PATH,       /**< Only file path */
-    COL_WIDTH_HEIGHT,    /**< String with width x height */
-    COL_WIDTH,           /**< Image width */
-    COL_HEIGHT,          /**< Image height */
-    NUM_COLS             /**< Number of columns */
-};
 /**
  * @brief  Structore to pass widgets and settings to callback
  */
@@ -91,26 +80,6 @@ get_pbuf_extension_list (void)
     return gsl_res;
 }
 /*----------------------------------------------------------------------------*/
-/**
- * @brief  Get file extenstion.
- *
- * @param[in] s_fn String with file path
- * @return    String with file extension
- */
-static char *
-get_file_ext (char *s_fn)
-{
-    char *s_ext = NULL;
-    char *s_p   = NULL;
-
-    s_p = strrchr (s_fn, '.');
-    if (s_p != NULL) {
-        s_p++;
-        s_ext = g_strdup (s_p);
-    }
-    return s_ext;
-}
-/*----------------------------------------------------------------------------*/
 /**
  * @brief  Check list with file names for GdkPixbuf images.
  *
@@ -147,512 +116,191 @@ check_files_for_pixbuf (GSList *gsl_files1)
 }
 /*----------------------------------------------------------------------------*/
 /**
- * @brief  Check list with file names for GdkPixbuf images.
+ * @brief  Get screen size.
  *
- * @param[in] s_path1 Path to directory
- * @return    List with files in directory
- */
-static GSList *
-get_directory_content (const char *s_path1)
-{
-    GSList *gsl_files = NULL;
-    char   *s_pthfn   = NULL;
-    char   *s_path    = NULL;
-    char   *s_fn      = NULL;
-    char   *s_p       = NULL;
-    int     i_dlen    = 0;
-    int     i_pos     = 0;
-    DIR    *dr;
-    struct  dirent *de;
-
-    i_dlen = strlen (s_path1);
-    s_path = g_malloc0 ((i_dlen + 1) * sizeof (char));
-    strcpy (s_path, s_path1);
-    s_p = strrchr (s_path, '/');
-
-    if (s_p != NULL) {
-        i_pos = (int) (s_p - s_path);
-        if (i_pos != i_dlen - 1) {
-            i_dlen++;
-            s_path = g_realloc (s_path, (i_dlen + 1) * sizeof (char));
-            strcat (s_path, "/");
-        }
-    }
-    dr = opendir (s_path); 
-    if (dr == NULL) {
-        printf ("Could not open current directory\n"); 
-        g_free (s_path);
-        return NULL; 
-    } 
-    while ((de = readdir(dr)) != NULL) 
-        //if (de->d_type == DT_REG) {
-        if (de->d_type == 8) {
-            s_fn = g_strdup (de->d_name);
-            s_pthfn = g_malloc0 ( 
-                    (i_dlen + strlen (s_fn) + 1) * sizeof (char));
-            strcpy (s_pthfn, s_path);
-            strcat (s_pthfn, s_fn);
-            gsl_files = g_slist_append (gsl_files, s_pthfn);
-            g_free (s_fn);
-        }
-    g_free (s_path);
-    closedir(dr);     
-    return gsl_files;
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Set data in GtkListStore.
- *
- * Sets data from ii_info ImageInfo to gls_list GtkListStore row pointed
- * by gti_iter GtkTreeIter.
- *
- * @param[out]  gls_list GtkListStore to set data
- * @param[in]   gti_iter GtkTreeIter row adress
- * @param[in]   ii_info  Data in ImageInfo object
- * @return  None
- */
-static void
-liststore_set_item (GtkListStore    *gls_list,
-                    GtkTreeIter     *gti_iter,
-                    const ImageInfo *ii_info)
-{
-    gtk_list_store_set (gls_list, gti_iter,
-                        COL_FULL_FILE_NAME, ii_info->s_full_path,
-                        COL_FILE_NAME,      ii_info->s_file_name,
-                        COL_FILE_PATH,      ii_info->s_file_path,
-                        COL_WIDTH_HEIGHT,   ii_info->s_width_height,
-                        COL_WIDTH,          ii_info->i_width,
-                        COL_HEIGHT,         ii_info->i_height,
-                        -1);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Insert single data item to GtkListStore.
- *
- * @param[out]  gw_list  GtkWidget with GtkListStore to insert data
- * @param[in]   ii_info  data in ImageInfo object
- * @return  None
- */
-static void
-liststore_add_item (GtkWidget       *gw_list,
-                    const ImageInfo *ii_info)
-{
-    GtkListStore *gls_store;
-    GtkTreeIter   gti_iter;
-
-    gls_store = GTK_LIST_STORE (gtk_tree_view_get_model (
-                GTK_TREE_VIEW (gw_list) ) );
-    gtk_list_store_append (gls_store, &gti_iter);
-    liststore_set_item (gls_store, &gti_iter, ii_info);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Insert multiple data items to GtkListStore.
- *
- * @param[out]  gw_list     GtkWidget with GtkListStore to insert items
- * @param[in]   gsl_iinfo1  GSList list with data in ImageInfo format
- * @return      None
- */
-static void
-liststore_add_items (GtkWidget  *gw_list,
-                     GSList     *gsl_iinfo1)
-{
-    GSList *gsl_iinfo = NULL;
-
-    gsl_iinfo = g_slist_copy (gsl_iinfo1);
-
-    while (gsl_iinfo != NULL) {
-        liststore_add_item (gw_list, gsl_iinfo->data);
-        gsl_iinfo = gsl_iinfo->next;
-    }
-    g_slist_free (gsl_iinfo);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Get data from GtkTreeModel to ImageInfo object.
- *
- * @param[in]  gtm_model GtkTreeModel with data to get
- * @param[in]  gti_iter  GtkTreeIter with row adress
- * @return     ImageInfo item with readed data
- */
-static ImageInfo *
-treemodel_get_data (GtkTreeModel *gtm_model,
-                    GtkTreeIter   gti_iter)
-{
-    ImageInfo *ii_info = g_malloc (sizeof (ImageInfo));
-
-    gtk_tree_model_get (gtm_model, &gti_iter,
-            COL_FULL_FILE_NAME, &ii_info->s_full_path, 
-            COL_FILE_NAME,      &ii_info->s_file_name,
-            COL_FILE_PATH,      &ii_info->s_file_path,
-            COL_WIDTH_HEIGHT,   &ii_info->s_width_height,
-            COL_WIDTH,          &ii_info->i_width,
-            COL_HEIGHT,         &ii_info->i_height, -1);
-    return ii_info;
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Find file on list and select it
- *
- * @param[in]   gw_tview  GtkTreeView on which data should be selected
- * @param[in]   s_file    File path to find
- * @return  None
- */
-static void
-find_select_item (GtkWidget  *gw_tview,
-                  const char *s_file)
-{
-    GtkTreeModel     *gtm_model;
-    GtkTreeSelection *gts_sele;
-    GtkTreeIter       gti_iter;
-    gboolean          b_res     = FALSE;
-    GValue            value     = {0,};
-
-    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (gw_tview));
-    b_res = gtk_tree_model_get_iter_first (gtm_model, &gti_iter);
-    while (b_res) {
-        gtk_tree_model_get_value(gtm_model, &gti_iter,
-                                 COL_FULL_FILE_NAME, &value);
-        const char *s_val = (const char*) g_value_get_string(&value);
-        if (compare_strings (s_file, s_val) == 0) {
-            gts_sele = gtk_tree_view_get_selection (GTK_TREE_VIEW (gw_tview));
-            gtk_tree_selection_select_iter (gts_sele, &gti_iter);
-            break;
-        }
-        g_value_unset(&value);
-        b_res = gtk_tree_model_iter_next (gtm_model, &gti_iter);
-    }
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Remove selected items from TreeView
- *
- * @param[out]  gw_tview  GtkTreeView from which data whould be removed
- * @return  None
- */
-static void
-treeview_remove_selected (GtkWidget *gw_tview)
-{
-    GList            *gl_list  = NULL;
-    GList            *gl_list1 = NULL;
-    GtkTreeSelection *gts_sele;
-    GtkTreeModel     *gtm_model;
-    GtkTreeIter       gti_iter;
-
-    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (gw_tview));
-    gts_sele = gtk_tree_view_get_selection (GTK_TREE_VIEW (gw_tview));
-    gl_list = gtk_tree_selection_get_selected_rows (gts_sele, &gtm_model);
-    gl_list1 = g_list_last (gl_list);
-
-    while (gl_list1 != NULL) {
-        if (gtk_tree_model_get_iter (gtm_model, &gti_iter, gl_list1->data)) {
-            gtk_list_store_remove (GTK_LIST_STORE (gtm_model), &gti_iter);
-            }
-        gl_list1 = gl_list1->prev;
-    }
-    g_list_free_full (gl_list, (GDestroyNotify) gtk_tree_path_free);
-    g_list_free (gl_list1);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Replace data in TreeView.
- *
- * @param[out]  gw_tview     TreeView in which data should be replaced
- * @param[in]   gsl_iinfo1   List with ImageInfo objects with data.
+ * @param[out]  i_scr_w  Screen width
+ * @param[out]  i_scr_h  Screen height
  * @return      none
  */
 static void
-treeview_replace_data (GtkWidget *gw_tview,
-                       GSList    *gsl_iinfo1)
+get_screen_size (int *i_scr_w,
+                 int *i_scr_h)
 {
-    GSList       *gsl_iinfo = NULL;
-    GtkTreeModel *gtm_model;
-    GtkListStore *gls_list;
-    GtkTreeIter   gti_iter;
-    uint32_t      ui_icnt   = 0;
-    uint32_t      ui_mcnt   = 0;
-    gboolean      b_res     = FALSE;
+    GdkRectangle workarea = {0};
 
-    gsl_iinfo = g_slist_copy (gsl_iinfo1);
+    gdk_monitor_get_workarea (
+        gdk_display_get_primary_monitor (gdk_display_get_default ()),
+        &workarea);
 
-    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (gw_tview));
-    gls_list = GTK_LIST_STORE (gtm_model);
-    ui_icnt = g_slist_length (gsl_iinfo);
-    ui_mcnt = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (gls_list), NULL);
-    
-    if (ui_icnt == ui_mcnt) {
-        b_res = gtk_tree_model_get_iter_first (gtm_model, &gti_iter);
-        while (b_res && gsl_iinfo != NULL) {
-            liststore_set_item (gls_list, &gti_iter, gsl_iinfo->data);
-            gsl_iinfo = gsl_iinfo->next;
-            b_res = gtk_tree_model_iter_next (gtm_model, &gti_iter);
-        }
+    *i_scr_w = workarea.width;
+    *i_scr_h = workarea.height;
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Paint a preview screen.
+ *
+ * @param[out]  gr_rect  GdkRectangle with area where preview image can be 
+ *                       painted
+ * @return      Pixbuf with preview screeen
+ */
+static GdkPixbuf *
+make_prev_screen_pbuf (GdkRectangle *gr_rect)
+{
+    GdkPixbuf *gp_bg;     /* Result whole preview image (screeb + wall) */
+    GdkPixbuf *gp_scr_1;  /* Preview (monitor) part 1 (top) image*/
+    GdkPixbuf *gp_scr_2;  /* Preview (monitor) part 1 (top) image*/
+    GdkPixbuf *gp_scr_3;  /* Preview (monitor) part 1 (top) image*/
+    int i_screen_w = 0;   /* Screen width */
+    int i_screen_h = 0;   /* Screen height */
+    int i_pr_scr_w = 138; /* Preview width (inside screen) */
+    int i_pr_scr_h = 0;   /* Preview height (inside screen) */
+    int i_img_w    = 0;   /* Preview (monitor) image width */
+    int i_img_1_h  = 0;   /* Preview (monitor) part 1 (top) image height */
+    int i_img_2_h  = 0;   /* Preview (monitor) part 2 (middle) image height */
+    int i_img_3_h  = 0;   /* Preview (monitor) part 3 (bottom) image height */
+
+    get_screen_size (&i_screen_w, &i_screen_h);
+
+    i_pr_scr_h = (int) ((double) i_screen_h /
+                        (double) i_screen_w * (double) i_pr_scr_w);
+
+    gr_rect->x = 7;               /* Preview left point (inside screen) */
+    gr_rect->y = 7;               /* Preview top point (inside screen) */
+    gr_rect->width = i_pr_scr_w;  /* Preview width (inside screen) */
+    gr_rect->height = i_pr_scr_h; /* Preview height (inside screen) */
+
+    gp_scr_1 = get_image (20);  /* top screen image (152x7) */
+    gp_scr_2 = get_image (21);  /* middle screen image (152x1) */
+    gp_scr_3 = get_image (22);  /* bottom screen image (152x45) */
+
+    i_img_w = gdk_pixbuf_get_width (gp_scr_1);
+    i_img_1_h = gdk_pixbuf_get_height (gp_scr_1);
+    i_img_2_h = gdk_pixbuf_get_height (gp_scr_2);
+    i_img_3_h = gdk_pixbuf_get_height (gp_scr_3);
+
+    gp_bg = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+                            TRUE,
+                            8,
+                            i_img_w,
+                            i_pr_scr_h + i_img_1_h + i_img_3_h);
+
+    gdk_pixbuf_scale (gp_scr_1,
+                      gp_bg,
+                      0,
+                      0,
+                      i_img_w,
+                      i_img_1_h,
+                      0.0,
+                      0.0,
+                      1.0,
+                      1.0,
+                      GDK_INTERP_HYPER);
+
+    for (int i = 0; i < i_pr_scr_h; ++i) {
+        gdk_pixbuf_scale (gp_scr_2,
+                          gp_bg,
+                          0,
+                          i_img_1_h + i,
+                          i_img_w,
+                          i_img_2_h,
+                          0.0,
+                          i_img_1_h + i,
+                          1.0,
+                          1.0,
+                          GDK_INTERP_HYPER);
     }
-    g_slist_free (gsl_iinfo);
+
+    gdk_pixbuf_scale (gp_scr_3,
+                      gp_bg,
+                      0,
+                      i_img_1_h + i_pr_scr_h,
+                      i_img_w,
+                      i_img_3_h,
+                      0.0,
+                      i_img_1_h + i_pr_scr_h,
+                      1.0,
+                      1.0,
+                      GDK_INTERP_HYPER);
+
+    g_object_unref (gp_scr_1);
+    g_object_unref (gp_scr_2);
+    g_object_unref (gp_scr_3);
+    return gp_bg;
 }
 /*----------------------------------------------------------------------------*/
 /**
- * @brief  Get data out of TreeView.
+ * @brief  Paint pixbuf on another pixbuf.
  *
- * @param[in]  gw_tview   TreeView from which data should be taken
- * @return     List with ImageInfo data of TreeView's TreeModel items
+ * @param[out]  gp_dest  Destination pixpuf to paint on to
+ * @param[in]   gp_src   Source pixpuf to paint on destination one
+ * @param[in]   gr_area  GdkRectangle with area where preview image can be 
+ *                       painted
+ * @return      none
  */
-static GSList *
-treeview_get_data (GtkWidget *gw_tview)
+static void
+paint_pbuf_on_pbuf (GdkPixbuf    *gp_dest,
+                    GdkPixbuf    *gp_src,
+                    GdkRectangle *gr_area)
 {
-    GSList       *gsl_iinfo = NULL;
-    GtkTreeModel *gtm_model;
-    GtkTreeIter   gti_iter;
-    gboolean      b_res     = FALSE;
+    double d_sc_x = 0;  /* x direction scale factor */
+    double d_sc_y = 0;  /* y direction scale factor */
 
-    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (gw_tview));
-    b_res = gtk_tree_model_get_iter_first (gtm_model, &gti_iter);
-    while (b_res) {
-        ImageInfo *ii_info = treemodel_get_data (gtm_model, gti_iter);
-        gsl_iinfo = g_slist_append (gsl_iinfo, ii_info);
-        b_res = gtk_tree_model_iter_next (gtm_model, &gti_iter);
+    d_sc_x = (double) gr_area->width / (double) gdk_pixbuf_get_width (gp_src);
+    d_sc_y = (double) gr_area->height / (double) gdk_pixbuf_get_height (gp_src);
+
+    gdk_pixbuf_scale (gp_src,
+                      gp_dest,
+                      gr_area->x,
+                      gr_area->y,
+                      gr_area->width,
+                      gr_area->height,
+                      gr_area->x,
+                      gr_area->y,
+                      d_sc_x, 
+                      d_sc_y,
+                      GDK_INTERP_HYPER);
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Make pixbuf preview of image from path.
+ *
+ * @param[in]  s_fname   Image path
+ * @return     Pixbuf with preview image.
+ */
+static GdkPixbuf *
+make_image_preview (const char *s_fname)
+{
+    GdkPixbuf *gp_prev_screen = NULL; /* Preview image (screen) */
+    GdkPixbuf *gp_prev = NULL;        /* Wallpaper preview image */
+    GdkRectangle gr_area = {0};       /* Wallpaper preview area */
+
+    gp_prev_screen = make_prev_screen_pbuf (&gr_area);
+    if (s_fname != NULL) {
+        gp_prev = gdk_pixbuf_new_from_file (s_fname, NULL);
+        paint_pbuf_on_pbuf (gp_prev_screen,
+                            gp_prev,
+                            &gr_area);
+        g_object_unref (gp_prev);
+    }   
+    return gp_prev_screen;
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Make preview image widget of image (file path).
+ *
+ * @param[in]   s_file       File to make preview image
+ * @param[out]  gw_img_prev  Preview image widget
+ * @return      none
+ */
+static void
+preview_from_file (const char *s_file,
+                   GtkWidget  *gw_img_prev)
+{
+    GdkPixbuf *gp_prev = make_image_preview (s_file);
+    if (gp_prev != NULL) {
+        gtk_image_set_from_pixbuf (GTK_IMAGE (gw_img_prev), gp_prev);
+        g_object_unref (gp_prev);
     }
-    return gsl_iinfo;
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Sort data in TreeView.
- *
- * @param[in,out]  gw_tview  TreeView in which data should be sorted
- * @return         none
- */
-static void
-treeview_sort_list (GtkWidget *gw_tview)
-{
-    GSList *gsl_files1 = NULL;
-    GSList *gsl_files  = NULL;
-
-    gsl_files1 = treeview_get_data (gw_tview);
-    gsl_files = g_slist_copy (gsl_files1);
-    gsl_files = g_slist_sort (gsl_files, (GCompareFunc) compare_imageitems);
-    treeview_replace_data (gw_tview, gsl_files);
-    g_slist_free_full (gsl_files1, (GDestroyNotify) imageinfo_free);
-    g_slist_free (gsl_files);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Move up selected items in TreeView.
- *
- * @param[in,out]  gw_tview  TreeView to move data up
- * @return         none
- */
-static void
-treeview_move_up (GtkWidget *gw_tview)
-{
-    GList            *gl_list  = NULL;
-    GList            *gl_list1 = NULL;
-    GtkTreeSelection *gts_sele;
-    GtkTreeModel     *gtm_model;
-    GtkTreeIter       gti_iter;
-    GtkTreeIter       gti_itern;
-
-    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (gw_tview));
-    gts_sele = gtk_tree_view_get_selection (GTK_TREE_VIEW (gw_tview));
-    gl_list = gtk_tree_selection_get_selected_rows (gts_sele, &gtm_model);
-
-    if (gl_list == NULL)
-        return;
-
-    gl_list1 = g_list_first (gl_list);
-
-    while (gl_list1 != NULL) {
-        if (gtk_tree_model_get_iter (gtm_model, &gti_iter, gl_list1->data)) {
-            gti_itern = gti_iter;
-            if (gtk_tree_model_iter_previous (gtm_model, &gti_itern)) {
-                gtk_list_store_swap (GTK_LIST_STORE (gtm_model),
-                                     &gti_iter, &gti_itern);
-            }
-            else break;
-        }
-        gl_list1 = gl_list1->next;
-    }
-    g_list_free_full (gl_list, (GDestroyNotify) gtk_tree_path_free);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Move down selected items in TreeView.
- *
- * @param[in,out]  gw_tview  TreeView to move data down
- * @return         none
- */
-static void
-treeview_move_down (GtkWidget *gw_tview)
-{
-    GList            *gl_list  = NULL;
-    GList            *gl_list1 = NULL;
-    GtkTreeSelection *gts_sele;
-    GtkTreeModel     *gtm_model;
-    GtkTreeIter       gti_iter;
-    GtkTreeIter       gti_itern;
-
-    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (gw_tview));
-    gts_sele = gtk_tree_view_get_selection (GTK_TREE_VIEW (gw_tview));
-    gl_list = gtk_tree_selection_get_selected_rows (gts_sele, &gtm_model);
-
-    if (gl_list == NULL)
-        return;
-
-    gl_list1 = g_list_last (gl_list);
-
-    while (gl_list1 != NULL) {
-        if (gtk_tree_model_get_iter (gtm_model, &gti_iter, gl_list1->data)) {
-            gti_itern = gti_iter;
-            if (gtk_tree_model_iter_next (gtm_model, &gti_itern)) {
-                gtk_list_store_swap (GTK_LIST_STORE (gtm_model),
-                                     &gti_iter, &gti_itern);
-            }
-            else break;
-        }
-        gl_list1 = gl_list1->prev;
-    }
-    g_list_free_full (gl_list, (GDestroyNotify) gtk_tree_path_free);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Add images button pressed.
- *
- * @param          widget   The object on which the signal is emitted
- * @param[in,out]  dd_data  DialogData object with settings and widget data
- * @return none
- */
-static void
-event_add_images_pressed (GtkWidget  *widget,
-                          DialogData *dd_data)
-{
-    GSList *gsl_files = NULL;
-    GSList *gsl_iinfo = NULL;
-
-    gsl_files = add_images_dialog (dd_data->gw_window);
-    gsl_iinfo = file_paths_to_imageinfo (gsl_files);
-
-    liststore_add_items (dd_data->gw_view, gsl_iinfo);
-    g_slist_free_full (gsl_files, g_free);
-    g_slist_free_full (gsl_iinfo, (GDestroyNotify) imageinfo_free);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Add images from folder button pressed.
- *
- * @param          widget   The object on which the signal is emitted
- * @param[in,out]  dd_data  DialogData object with settings and widget data
- * @return none
- */
-static void
-event_add_images_folder_pressed (GtkWidget  *widget,
-                                 DialogData *dd_data)
-{
-    GSList *gsl_files1 = NULL;
-    GSList *gsl_files2 = NULL;
-    char   *s_folder   = NULL;
-
-    s_folder = add_images_folder_dialog (dd_data->gw_window);
-
-    if (s_folder != NULL) {
-        gsl_files1 = get_directory_content (s_folder);
-        gsl_files2 = check_files_for_pixbuf (gsl_files1);
-        g_slist_free_full (gsl_files1, g_free);
-        gsl_files1 = file_paths_to_imageinfo (gsl_files2);
-        liststore_add_items (dd_data->gw_view, gsl_files1);
-        g_slist_free_full (gsl_files1, (GDestroyNotify) imageinfo_free);
-        g_slist_free_full (gsl_files2, g_free);
-        g_free (s_folder);
-    }
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Remove items from list pressed.
- *
- * @param          widget    The object on which the signal is emitted
- * @param[in,out]  gw_tview  TreeView to remove items
- * @return none
- */
-static void
-event_remove_from_list_pressed (GtkWidget *widget,
-                                GtkWidget *gw_tview)
-{
-    treeview_remove_selected (gw_tview);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Move up items pressed.
- *
- * @param          widget    The object on which the signal is emitted
- * @param[in,out]  gw_tview  TreeView to move items up
- * @return none
- */
-static void
-event_move_up_pressed (GtkWidget *widget,
-                       GtkWidget *gw_tview)
-{
-    treeview_move_up (gw_tview);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Move down items pressed.
- *
- * @param          widget    The object on which the signal is emitted
- * @param[in,out]  gw_tview  TreeView to move items down
- * @return none
- */
-static void
-event_move_down_pressed (GtkWidget *widget,
-                         GtkWidget *gw_tview)
-{
-    treeview_move_down (gw_tview);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Sort wallpaper list button pressed.
- *
- * @param          widget    The object on which the signal is emitted
- * @param[in,out]  gw_tview  TreeView to sort items
- * @return none
- */
-static void
-event_sort_list_pressed (GtkWidget *widget,
-                         GtkWidget *gw_tview)
-{
-    treeview_sort_list (gw_tview);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Set wallpaper button pressed.
- *
- * @param          widget   The object on which the signal is emitted
- * @param[in,out]  dd_data  DialogData object with settings and widget data
- * @return none
- */
-static void
-event_set_wallpaper_pressed (GtkWidget  *widget,
-                             DialogData *dd_data)
-{
-    GList            *gl_list  = NULL;
-    GtkTreeSelection *gts_sele;
-    GtkTreeModel     *gtm_model;
-    GtkTreeIter       gti_iter;
-
-    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (dd_data->gw_view));
-    gts_sele = gtk_tree_view_get_selection (GTK_TREE_VIEW (dd_data->gw_view));
-    gl_list = gtk_tree_selection_get_selected_rows (gts_sele, &gtm_model);
-
-    if (gl_list == NULL)
-        return;
-
-    if (gtk_tree_model_get_iter (gtm_model, &gti_iter, gl_list->data)) {
-        ImageInfo *ii_info = treemodel_get_data (gtm_model, gti_iter);
-        printf ("%s\n", ii_info->s_full_path);
-
-        if (wallpaper_dialog_set (dd_data->ws_sett,ii_info->s_full_path) > 0) {
-            printf ("Some error occured\n");
-        }
-        imageinfo_free (ii_info);
-    }
-    g_list_free_full (gl_list, (GDestroyNotify) gtk_tree_path_free);
 }
 /*----------------------------------------------------------------------------*/
 /**
@@ -708,6 +356,147 @@ gather_settings (DialogData *dd_data)
     settings_set_command (dd_data->ws_sett, s_cmd);
 }
 /*----------------------------------------------------------------------------*/
+/**
+ * @brief  Add images button pressed.
+ *
+ * @param          widget   The object on which the signal is emitted
+ * @param[in,out]  dd_data  DialogData object with settings and widget data
+ * @return         none
+ */
+static void
+event_add_images_pressed (GtkWidget  *widget,
+                          DialogData *dd_data)
+{
+    GSList *gsl_files = NULL;
+    GSList *gsl_iinfo = NULL;
+
+    gsl_files = add_images_dialog (dd_data->gw_window);
+    gsl_iinfo = file_paths_to_imageinfo (gsl_files);
+
+    liststore_add_items (dd_data->gw_view, gsl_iinfo);
+    g_slist_free_full (gsl_files, g_free);
+    g_slist_free_full (gsl_iinfo, (GDestroyNotify) imageinfo_free);
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Add images from folder button pressed.
+ *
+ * @param          widget   The object on which the signal is emitted
+ * @param[in,out]  dd_data  DialogData object with settings and widget data
+ * @return         none
+ */
+static void
+event_add_images_folder_pressed (GtkWidget  *widget,
+                                 DialogData *dd_data)
+{
+    GSList *gsl_files1 = NULL;
+    GSList *gsl_files2 = NULL;
+    char   *s_folder   = NULL;
+
+    s_folder = add_images_folder_dialog (dd_data->gw_window);
+
+    if (s_folder != NULL) {
+        gsl_files1 = get_directory_content (s_folder);
+        gsl_files2 = check_files_for_pixbuf (gsl_files1);
+        g_slist_free_full (gsl_files1, g_free);
+        gsl_files1 = file_paths_to_imageinfo (gsl_files2);
+        liststore_add_items (dd_data->gw_view, gsl_files1);
+        g_slist_free_full (gsl_files1, (GDestroyNotify) imageinfo_free);
+        g_slist_free_full (gsl_files2, g_free);
+        g_free (s_folder);
+    }
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Remove items from list pressed.
+ *
+ * @param          widget    The object on which the signal is emitted
+ * @param[in,out]  gw_tview  TreeView to remove items
+ * @return         none
+ */
+static void
+event_remove_from_list_pressed (GtkWidget *widget,
+                                GtkWidget *gw_tview)
+{
+    treeview_remove_selected (gw_tview);
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Move up items pressed.
+ *
+ * @param          widget    The object on which the signal is emitted
+ * @param[in,out]  gw_tview  TreeView to move items up
+ * @return         none
+ */
+static void
+event_move_up_pressed (GtkWidget *widget,
+                       GtkWidget *gw_tview)
+{
+    treeview_move_up (gw_tview);
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Move down items pressed.
+ *
+ * @param          widget    The object on which the signal is emitted
+ * @param[in,out]  gw_tview  TreeView to move items down
+ * @return         none
+ */
+static void
+event_move_down_pressed (GtkWidget *widget,
+                         GtkWidget *gw_tview)
+{
+    treeview_move_down (gw_tview);
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Sort wallpaper list button pressed.
+ *
+ * @param          widget    The object on which the signal is emitted
+ * @param[in,out]  gw_tview  TreeView to sort items
+ * @return         none
+ */
+static void
+event_sort_list_pressed (GtkWidget *widget,
+                         GtkWidget *gw_tview)
+{
+    treeview_sort_list (gw_tview);
+}
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief  Set wallpaper button pressed.
+ *
+ * @param          widget   The object on which the signal is emitted
+ * @param[in,out]  dd_data  DialogData object with settings and widget data
+ * @return         none
+ */
+static void
+event_set_wallpaper_pressed (GtkWidget  *widget,
+                             DialogData *dd_data)
+{
+    GList            *gl_list  = NULL;
+    GtkTreeSelection *gts_sele;
+    GtkTreeModel     *gtm_model;
+    GtkTreeIter       gti_iter;
+
+    gtm_model = gtk_tree_view_get_model (GTK_TREE_VIEW (dd_data->gw_view));
+    gts_sele = gtk_tree_view_get_selection (GTK_TREE_VIEW (dd_data->gw_view));
+    gl_list = gtk_tree_selection_get_selected_rows (gts_sele, &gtm_model);
+
+    if (gl_list == NULL)
+        return;
+
+    if (gtk_tree_model_get_iter (gtm_model, &gti_iter, gl_list->data)) {
+        ImageInfo *ii_info = treemodel_get_data (gtm_model, gti_iter);
+
+        if (wallpaper_dialog_set (dd_data->ws_sett,ii_info->s_full_path) > 0) {
+            printf ("Some error occured\n");
+        }
+        imageinfo_free (ii_info);
+    }
+    g_list_free_full (gl_list, (GDestroyNotify) gtk_tree_path_free);
+}
+/*----------------------------------------------------------------------------*/
 /**
  * @brief  Save settings button pressed.
  *
@@ -723,62 +512,6 @@ event_save_settings_pressed (GtkWidget  *widget,
     settings_write (dd_data->ws_sett);
 }
 /*----------------------------------------------------------------------------*/
-/**
- * @brief  Make pixbuf preview of image from path.
- *
- * @param[in]  s_fname   Image path
- * @param[in]  i_max_wh  Max preview width/height 
- * @return  Pixbuf with preview image.
- */
-static GdkPixbuf *
-make_image_preview (const char *s_fname,
-                    const int   i_max_wh)
-{
-    GdkPixbuf *gp_prev        = NULL;
-    GdkPixbuf *gp_prev_scaled = NULL;
-    float      f_w            = 0;
-    float      f_h            = 0;
-    float      f_nw           = 0;
-    float      f_nh           = 0;
-
-    gp_prev = gdk_pixbuf_new_from_file (s_fname, NULL);
-
-    if (gp_prev != NULL) {
-        f_w = (float) gdk_pixbuf_get_width (gp_prev);
-        f_h = (float) gdk_pixbuf_get_height (gp_prev);
-        if (f_w > f_h) {
-            f_nw = (float) i_max_wh; 
-            f_nh = f_h / f_w * f_nw;
-        }
-        else {
-            f_nh = (float) i_max_wh; 
-            f_nw = f_w / f_h * f_nh;
-        }
-        gp_prev_scaled = gdk_pixbuf_scale_simple (gp_prev,
-                (int) f_nw, (int) f_nh, GDK_INTERP_HYPER);
-    }
-    g_object_unref (gp_prev);
-    return gp_prev_scaled;
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Make preview image widget of image (file path).
- *
- * @param[in]   s_file       File to make preview image
- * @param[out]  gw_img_prev  Preview image widget
- * @return      none
- */
-static void
-preview_from_file (const char *s_file,
-                   GtkWidget  *gw_img_prev)
-{
-    GdkPixbuf *gp_prev = make_image_preview (s_file, 150);
-    if (gp_prev != NULL) {
-        gtk_image_set_from_pixbuf (GTK_IMAGE (gw_img_prev), gp_prev);
-        g_object_unref (gp_prev);
-    }
-}
-/*----------------------------------------------------------------------------*/
 /**
  * @brief  Make preview image widget of image (file path).
  *
@@ -852,13 +585,13 @@ event_on_delete (GtkWidget  *window,
 static void
 create_title_widget (GtkWidget **gw_title_widget)
 {
-    *gw_title_widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-
-    GtkWidget *gw_label = gtk_label_new (NULL);
-    const char *s_str = "Wall Changer - Program settings";
+    const char *s_str    = "Wall Changer - Program settings";
     const char *s_format = "<span size=\"20000\" weight=\"bold\" \
-                          foreground=\"#0099e6\" style=\"italic\">\%s</span>";
-    char *s_markup;
+                            foreground=\"#0099e6\" style=\"italic\">\%s</span>";
+    char       *s_markup = NULL;
+    GtkWidget  *gw_label = gtk_label_new (NULL);
+
+    *gw_title_widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
     s_markup = g_markup_printf_escaped (s_format, s_str);
     gtk_label_set_markup (GTK_LABEL (gw_label), s_markup);
     g_free (s_markup);
@@ -900,50 +633,6 @@ create_image_button (const char *s_label,
     return gw_btn;
 }
 /*----------------------------------------------------------------------------*/
-/**
- * @brief  Create treeview for image list.
- *
- * @param[out]  gw_tview  Pointer to Treeview to set
- * @return      none
- */
-static void
-create_tview (GtkWidget **gw_tview)
-{
-    GtkCellRenderer  *gcr_render;
-    GtkListStore     *gls_list;
-    GtkTreeSelection *gts_sele;
-
-    *gw_tview = gtk_tree_view_new ();
-    gts_sele = gtk_tree_view_get_selection (GTK_TREE_VIEW (*gw_tview));
-    gtk_tree_selection_set_mode (gts_sele, GTK_SELECTION_MULTIPLE);
-    gcr_render = gtk_cell_renderer_text_new ();
-    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (*gw_tview),
-                                                 -1,
-                                                 "File name",
-                                                 gcr_render,
-                                                 "text",
-                                                 COL_FILE_NAME,
-                                                 NULL);
-    gcr_render = gtk_cell_renderer_text_new ();
-    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (*gw_tview),
-                                                 -1,
-                                                 "Dim",
-                                                 gcr_render,
-                                                 "text",
-                                                 COL_WIDTH_HEIGHT,
-                                                 NULL);
-    gls_list = gtk_list_store_new (NUM_COLS,
-                                   G_TYPE_STRING,
-                                   G_TYPE_STRING,
-                                   G_TYPE_STRING,
-                                   G_TYPE_STRING,
-                                   G_TYPE_INT,
-                                   G_TYPE_INT);
-    gtk_tree_view_set_model (GTK_TREE_VIEW (*gw_tview),
-                             GTK_TREE_MODEL (gls_list));
-    g_object_unref(gls_list);
-}
-/*----------------------------------------------------------------------------*/
 /**
  * @brief  Make default empty preview image.
  *
@@ -951,13 +640,9 @@ create_tview (GtkWidget **gw_tview)
  * @return      none
  */
 static void
-create_preview (GtkWidget **gw_img)
+create_default_preview (GtkWidget **gw_img)
 {
-    GdkPixbuf *gp_prev = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-                                         TRUE,
-                                         8,
-                                         150,
-                                         150);
+    GdkPixbuf *gp_prev = make_image_preview (NULL);
     if (gp_prev != NULL) {
         *gw_img = gtk_image_new_from_pixbuf (gp_prev);
         g_object_unref (gp_prev);
@@ -1059,7 +744,7 @@ create_buttons_widget (GtkWidget **gw_buttons_widget,
  * @param[out]     gw_settings_widget  Pointer to destination widget
  * @param[in,out]  dd_data             DialogData object with settings and
  *                                     widget data
- * @return  none
+ * @return         none
  */
 static void
 create_settings_widget (GtkWidget **gw_settings_widget,
@@ -1129,18 +814,18 @@ static void
 activate (GtkApplication *app,
           DialogData     *dd_data)
 {
-    GtkWidget     *gw_window;
-    GtkWidget     *gw_title_widget;
-    GtkWidget     *gw_tview;
-    GtkWidget     *gw_scroll;
-    GtkWidget     *gw_buttons_widget;
-    GtkWidget     *gw_box_list_btns;
-    GtkWidget     *gw_box_prev;
-    GtkWidget     *gw_img_prev;
-    GtkWidget     *gw_settings_widget;
+    GtkWidget     *gw_window;          // Application window
+    GtkWidget     *gw_title_widget;    // Top title widget
+    GtkWidget     *gw_tview;           // Wallpaper list TreeView
+    GtkWidget     *gw_scroll;          // Scrolled window for wallpaper list
+    GtkWidget     *gw_buttons_widget;  // Buttons widget
+    GtkWidget     *gw_img_prev;        // Wallpaper preview widget
+    GtkWidget     *gw_box_prev;        // Widget for wallpaper preview
+    GtkWidget     *gw_box_list_btns;   // Widget for list, buttons, preview
+    GtkWidget     *gw_settings_widget; // Setings for wallpaper changing
 
     gw_window = gtk_application_window_new (app);
-    gtk_window_set_title (GTK_WINDOW (gw_window), "Wall Changer v1.0.1");
+    gtk_window_set_title (GTK_WINDOW (gw_window), "Wall Changer v1.1");
     gtk_window_set_default_size (GTK_WINDOW (gw_window), 1024, 768);
     g_signal_connect (gw_window, "delete-event",
                   G_CALLBACK (event_on_delete), dd_data);
@@ -1148,7 +833,7 @@ activate (GtkApplication *app,
 
     create_title_widget (&gw_title_widget);
 
-    create_preview (&gw_img_prev);
+    create_default_preview (&gw_img_prev);
 
     create_tview (&gw_tview);
     dd_data->gw_view = gw_tview;
@@ -1169,6 +854,7 @@ activate (GtkApplication *app,
     gtk_box_pack_start (GTK_BOX (gw_box_prev),
                         gw_img_prev,
                         FALSE, FALSE, 4);
+
     gtk_box_pack_start (GTK_BOX (gw_box_list_btns),
                         gw_scroll,
                         TRUE, TRUE, 4);
@@ -1218,8 +904,6 @@ activate (GtkApplication *app,
 
     find_select_item (gw_tview, dd_data->ws_sett->s_lastused);
 
-    //wallpaper_startup_set (dd_data->ws_sett);
-
     gtk_window_set_application (GTK_WINDOW (gw_window), GTK_APPLICATION (app));
     gtk_widget_show_all (gw_window);
 }
